/** 
    @example  I2C Slave Demo code
    @brief I2C Slave Basic demo code.
    Description: Stores Bytes received from Master
        - returns same bytes when read request is received
    @version  V0.1
    @author   ADI
    @date     August 2017

 *****************************************************************************

All files for ADuCM360/361 provided by ADI, including this file, are
provided  as is without warranty of any kind, either expressed or implied.
The user assumes any and all risk from the use of this code.
It is the responsibility of the person integrating this code into an application
to ensure that the resulting application performs as required and is safe.

**/

#include <stdio.h>
#include <string.h>
#include <ADuCM360.h>

#include <I2cLib.h>
#include <ClkLib.h>
#include <WdtLib.h>
#include <IntLib.h>
#include <DioLib.h>
#include <UrtLib.h>

// Function Definitions
void delay(long int);                             // delay function
void I2CSlaveSetup(void);                         // I2C Slave setup function 

unsigned char wr_byte_addr0 = 0x00;               // Used to address Array

unsigned char  A0h[256], A2h[256];                // Arrays 
unsigned char  i2c_first = 1;                     // Staus flag for incoming I2C packet

unsigned char ucTxCnt = 0;                        // Counts number of I2C bytes transmitted
unsigned char ucRxCnt =0;                         // Counts number of I2C bytes received
unsigned char ucEmptyCnt = 0;
unsigned char ucStopCnt =0;
unsigned char ucTestExtraRead = 0;
unsigned char ucFirstDataWrite = 0;

int main (void)
{
    int i = 0;
    WdtCfg(T3CON_PRE_DIV1,T3CON_IRQ_EN,
        T3CON_PD_DIS);                              // Disable Watchdog timer resets
    //Disable clock to unused peripherals
    ClkDis(CLKDIS_DISPWMCLK|CLKDIS_DISSPI1CLK|      // disable unwanted clocks
        CLKDIS_DISDACCLK|CLKDIS_DISDMACLK|
        CLKDIS_DISADCCLK);
    ClkCfg(CLK_CD0,CLK_HF,
        CLKSYSDIV_DIV2EN_DIS,CLK_UCLKCG);           // Select CD0 for CPU clock - 16Mhz clock
    ClkSel(CLK_CD7,CLK_CD0,CLK_CD7,CLK_CD7);        // Select CD0 for I2C System clock
    DioCfg(pADI_GP0,0x9028);                        // Configure P0.1/0.2 as I2C and P0.6/0.7 as UART pins
    I2CSlaveSetup();                                // Call function to initialize I2C Slave

    DioOenPin(pADI_GP1,PIN3,1);                     // Configure P1.3 as an output - to toggle LED with Timer 2
    DioSet(pADI_GP1, BIT3);
    // Initialize table    
    for(i=0;i<256;i++)                              // Initialize memory values to 0
    {
        A0h[i]=i;
        A2h[i]=(255-i);
    }
    i2c_first = 1;
    I2cFifoFlush(SLAVE,ENABLE);                      // Flush the I2C FIFOs on startup
    while (1)
    {
          pADI_GP1->GPTGL = 0x8;                     // Toggle P1.3
      delay(0x60000);                                // Delay routine
     }
}
/// Function used to setup the I2C0 as Slave
void I2CSlaveSetup(void)
{
    I2cSIDCfg(0xA0,0xA6,0xA8,0xA0);                  // Configure Slave ID value;           
    I2cFifoFlush(SLAVE,ENABLE);                      // Enable flush of Slave FIFOs
    I2cFifoFlush(SLAVE,DISABLE);
    I2cSCfg(0,
    I2CSCON_IENREPST_EN|I2CSCON_IENTX_EN|
    I2CSCON_IENRX_EN|I2CSCON_IENSTOP_EN,
    I2CSCON_SLV_EN|I2CSCON_EARLYTXR_EN);             // Enable Slave mode - all interrupt sources
    NVIC_EnableIRQ(I2CS_IRQn);                       // Enable I2C Slave interrupt source
}

void I2C0_Slave_Int_Handler()
{
    unsigned int I2CSSTATUS = 0;
    volatile unsigned char ucTest = 0;
    I2CSSTATUS = I2cSta(SLAVE);                       // read I2C Slave status
    if ((I2CSSTATUS & BIT3) == BIT3)                  // If I2C Slave Rx IRQ
    {
        A0h[wr_byte_addr0] = I2cRx(SLAVE);            // get byte data
        wr_byte_addr0++;
        ucFirstDataWrite = 0;                       
    }
    if ((I2CSSTATUS & 0x4) == 0x4)                    // If I2C Master Tx IRQ
    {
        I2cTx(SLAVE,A0h[wr_byte_addr0++]);            // Load Tx FIFO
    }   
    if ((I2CSSTATUS & 0x2) == 0x2)                    // Slave Transmit underflow
    {
        I2cTx(SLAVE,A0h[wr_byte_addr0++]);            // Load Tx FIFO
    }
    if ((I2CSSTATUS & 0x10) == 0x10)                  // Slave receive overflow
    {
        ucTest++;
    }
    if ((I2CSSTATUS & 0x20) == 0x20)                  // If I2C NOACK
    {
        ucTest++;
    }
    if ((I2CSSTATUS & 0x80) == 0x80)                  // I2C General Call Interrupt
    {

    }
    if ((I2CSSTATUS & 0x400) == 0x400)                // If I2C Stop after Start condition detected IRQ
    {
        i2c_first = 1;
        I2cFifoFlush(SLAVE,ENABLE);                   // Enable flush of Slave FIFOs
        I2cFifoFlush(SLAVE,DISABLE);
        ucStopCnt++; 
        wr_byte_addr0 = 0;
    }
    if ((I2CSSTATUS & 0x2000) == 0x2000)              // If I2C Repeated Start detected IRQ
    {
    }
}

// Simple Delay routine
void delay (long int length)
{
    while (length >0)
        length--;
}




